#   Analytical toy-model for studying Graphene/SAM/Au devices
#
#   Authors of this code :  M. Famili, I. Grace, C. J. Lambert
#
#   Please site the refering article in all your publications arising
#   from your use of this code
#
import numpy as np
import matplotlib.pyplot as plt
import scipy as sp
from scipy import integrate
def fi_loc(array,value):
	idx = (np.abs(array-value)).argmin()
	return idx


ED0=-0.2
Gam0=0.1
Gam1=0.5
em=1
z=1

pref=7.74E-5
alph=1
gD=1
gG=0
bD=0
bG=0
dE=0.001
dV=0.05

VD=np.arange(-1,1.1,dV)
VG=np.arange(-1,1,2)
E=np.arange(-2,2,dE)
EDir=np.zeros((len(VD),len(VG)))
epsi=np.zeros((len(VD),len(VG)))
Gam2=np.zeros(len(E))
T=np.zeros((len(E),len(VD),len(VG)))
int_cur=np.zeros((len(VD),len(VG)))
dif_cur=np.zeros((len(VD),len(VG)))
Eplot=np.zeros((len(E),len(VG)))
for v_d in range(len(VD)):
	for v_g in range(len(VG)):
		EDir[v_d,v_g]=ED0-gD*VD[v_d]-gG*VG[v_g]
		epsi[v_d,v_g]=em-bD*VD[v_d]-bG*VG[v_g]	

for v_g in range(len(VG)):
	print(v_g)
	for v_d in range(len(VD)):
		for e in range(len(E)):
			Gam2[e]=Gam0+alph*abs(E[e]-EDir[v_d,v_g])**z
			T[e,v_d,v_g]=4*Gam1*Gam2[e]/((E[e]-epsi[v_d,v_g])**2+(Gam1+Gam2[e]))**2	
		Tint=T[min(fi_loc(E,-VD[v_d]),fi_loc(E,0)):max(fi_loc(E,-VD[v_d]),fi_loc(E,0)),v_d,v_g]
		int_cur[v_d,v_g]=(sp.integrate.trapz(T[min(fi_loc(E,-VD[v_d]),fi_loc(E,0)):max(fi_loc(E,-VD[v_d]),fi_loc(E,0)),v_d,v_g],dx=dE)*VD[v_d]/abs(VD[v_d]))*pref
		dif_cur[:,v_g]=np.gradient(int_cur[:,v_g],dV)
		Eplot[:,v_g]=E
Tplot=T[:,::10,:]
VDplot=VD[::10]

plt.figure(1)
plt.subplot(212)
for v_g in range(len(VG)):
	for v_d in range(len(VDplot)):
			plt.plot(Eplot,Tplot[:,v_d,:],label='$V_D=$'+str(round(-VDplot[v_d],2)),linewidth=1)
plt.ylabel('$T(E)$')
plt.xlabel('$E$')
plt.legend()


plt.subplot(221)
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
plt.ylabel('$I$')
plt.xlabel('$V_D(V)$')
for v_g in range(len(VG)):
    plt.plot(VD,int_cur[:,v_g],label='$V_G=$'+str(round(-VG[v_g],2)),linewidth=1)
plt.legend()


plt.subplot(222)
plt.ticklabel_format(style='sci', axis='y', scilimits=(0,0))
for v_g in range(len(VG)):
	plt.plot(VD,dif_cur[:,v_g], label='$V_G=$'+str(round(-VG[v_g],2)),linewidth=1)
plt.legend()
plt.ylabel('$dI/dV$')
plt.xlabel('$V_D(V)$')
plt.draw()
plt.show()


